# -*- coding: utf-8 -*-
from ladon.exceptions.types import NonUnicodeError, FromUnicodeConversionError, UnsafeConversionDisabled, UnsafeConversionError
from ladon.compat import PORTABLE_BYTES, PORTABLE_STRING, safe_conversions


class TypeConverter(object):
	"""The type converter mainly serves as a tool for the dispatcher providing the means to
	validate and convert between the interface and function layers
	"""

	def __init__(self, encoding='UTF-8', allow_unsafe_conversion=False, only_strings_to_unicode=True):
		"""In the TypeConverter constructor you can set conversion properties like which
		encoding to use when converting forth and back between unicode to str (or bytes and
		str for python 3). The default encoding is UTF-8.
		allow_unsafe_conversion is a boolean that tells the TypeConverter if unsafe conversions are allowed,
		An example of an unsafe conversion is float to int, as any floating point part will be truncated
		in the conversion.
		only_strings_to_unicode tells the converter to let types other than unicode and str (or bytes and
		str in python 3) be converted to the expected type when calling to_unicode_string
		"""
		# Converter options
		self.encoding = encoding
		self.allow_unsafe_conversion = allow_unsafe_conversion
		self.only_strings_to_unicode = only_strings_to_unicode

	def from_unicode_string(self, val, typ):
		"""Convert a unicode string to certain types.
		"""
		# Check the value type
		if not isinstance(val, PORTABLE_STRING):
			raise NonUnicodeError(type(val))

		# unicode to unicode or raw string
		if typ in (PORTABLE_BYTES, PORTABLE_STRING):
			if typ is PORTABLE_STRING:
				return val
			elif typ is PORTABLE_BYTES:
				return val.encode(self.encoding)

		# other primitive types
		try:
			if typ is bool and (val[0].upper() == PORTABLE_STRING('F') or val.strip() == '0'):
				return False
			return typ(val)
		except Exception as e:
			raise FromUnicodeConversionError(typ, str(e))

	def to_unicode_string(self, val, expected_typ):
		"""Convert a primitive type to unicode. The method uses TypeConverter
		options in the decition making, and encoding.
		"""
		# TODO: Should we check?
		# isinstance(val, expected_typ)

		# Expecting unicode or raw string
		if expected_typ in (PORTABLE_BYTES, PORTABLE_STRING):
			# No unsafe conversion can enter here as long as the user's service method obeys
			# the internal encoding it advertises for raw strings (aka bytes in python 3)
			if isinstance(val, PORTABLE_STRING):
				return val
			elif isinstance(val, PORTABLE_BYTES):
				return PORTABLE_STRING(val, self.encoding)
			else:
				return PORTABLE_STRING(val)

		if not self.allow_unsafe_conversion:
			# Check if the conversion is unsafe
			if expected_typ not in safe_conversions[typ]:
				raise UnsafeConversionDisabled(typ, expected_typ)
		try:
			val = expected_typ(val)
		except Exception as e:
			raise UnsafeConversionError(typ, expected_typ, str(e))

		if self.only_strings_to_unicode:
			return val
		else:
			return PORTABLE_STRING(val)
